#ifndef SENTECSENSORS_H
#define SENTECSENSORS_H

#include <Arduino.h>
#include <ErrorTypes.h>

class SentecSensorRS485 {
public:
    byte address;
    uint8_t serialCommunicationControlPin = 19;
    HardwareSerial *RS485;
    byte answerFrame[10];
    // TODO use valid flag to log None
    bool valid = false;

    SentecSensorRS485(HardwareSerial *ser, byte add);

    SentecSensorRS485(HardwareSerial *ser, byte add, uint8_t serialControlPin);

    void write(byte queryFrame[], int length);

    String getValueStr(float value);

    String getValueStr(int value);

    ErrorType queryAddress();

    ErrorType readRegister(int registerStartAddress);

    ErrorType readRegister(int registerStartAddress, int registerLength);

    ErrorType writeRegister(int registerAddress, int value);

    ErrorType setAddress(byte add);

    void resetAnswerFrame();

    ErrorType getResponse();

    unsigned int calculateCRC(byte query[], int length);

    void printBytes(byte *data, int length);

    void printBytes(word data);

    std::string formatBytes(byte *data, int length);

    std::string formatBytes(word data);

};

class SolarRadiationSensor : public SentecSensorRS485 {
public:
    using SentecSensorRS485::SentecSensorRS485;
    // global radiation [W/m^2]
    word glob = 0;

    word getSolarRadiation();

    String getSolarRadiationStr();
};

class RainGaugeSensor : public SentecSensorRS485 {
public:
    using SentecSensorRS485::SentecSensorRS485;
    // precipitation values [mm]
    word precipitation = 0; // prcp since reset? (I THINK!!!)

    void resetPrecipitation();

    void resetSensor();

    word getInstantaneousPrecipitation();

    String getPrecipitationStr();
};

class SoilMoistureSensor : public SentecSensorRS485 {
public:
    using SentecSensorRS485::SentecSensorRS485;
    // vwc: volumetric water content [%]
    uint16_t moistureRaw = 0;
    int moistureOffset = 0;
    // soil temperature [deg C]
    int temperatureRaw = 0;
    int temperatureOffset = 0;

    float getMoistureTemp();

    float getMoisture();

    String getMoistureStr();

    String getTemperatureStr();
};

#endif
