#include "ram_caching.hpp"

static const char* TAG = "CACHING";
RTC_DATA_ATTR char storage[maxLength_][maxSize_];
RTC_DATA_ATTR int storedElements;
RTC_DATA_ATTR int headElement = 0;
RTC_DATA_ATTR int tailElement = 0;

void RtcMemory::store_data(std::string message){

    // turn data into char array
    const char* jsonString = message.c_str();

    // move head to new element
    headElement = (headElement + 1) % maxSize_;
    // apparently I am not allowed to copy to rtc mem
    for(int i=0; i<maxLength_; i++){
        storage[i][headElement] = jsonString[i];
        if(jsonString[i] == '\0'){
            break;
        }
    }
    storedElements++;
    ESP_LOGE(TAG, "Moved message to storage.");
}

String RtcMemory::get_data_from_storage(){
    // remove element pointed at by tail
    String buf = "";
    char current = '\0';
    for(int i = 0; i<maxLength_; i++){
        current = storage[i][tailElement];
        buf += current;
        if(current == '\0'){
            break;
        }
    }
    // move tail to next element
    tailElement = (tailElement + 1) % maxSize_;
    storedElements--;
    ESP_LOGE(TAG, "Retrieved message from storage");
    return buf;
}

bool RtcMemory::is_data_storage_full(){
    return headElement == tailElement;
}

int RtcMemory::amount_stored_data(){
    return storedElements;
}

void RtcMemory::get_host_mac(uint8_t* destination, bool open){
    
    if(!open){preferences.begin("config", true);}

	if (preferences.isKey("host")) {
		preferences.getBytes("host", destination, sizeof(uint8_t) * 6);
        ESP_LOGI(TAG, "Host Mac retrieved from flash");
	} else {
		memcpy(destination, BROADCAST_MAC, sizeof(BROADCAST_MAC));
		ESP_LOGI(TAG, "Backup MAC address used");
	}

	if(!open){preferences.end();}

}

void RtcMemory::store_mac_address(uint8_t* mac){
    preferences.begin("config", false);
    if(preferences.putBytes("host", mac, sizeof(uint8_t) * 6) > 0){
        ESP_LOGI(TAG, "Host MAC address saved to flash %02X:%02X:%02X:%02X:%02X:%02X", mac[0],
        mac[1],mac[2],mac[3],mac[4],mac[5]);
        }
        else{
            ESP_LOGI(TAG, "Couldn't save Host Mac to flash");
        }
    preferences.end();
}

bool RtcMemory::does_host_exist(){
    preferences.begin("config", true);
    bool answer = preferences.isKey("host");
    preferences.end();
    return answer;
}