#include "f_deep_sleep.hpp"

namespace DeepSleep {
static const std::string TAG = "DEEP_SLEEP";

void print_wakeup_reason()
{
	esp_sleep_wakeup_cause_t wakeup_reason;

	wakeup_reason = esp_sleep_get_wakeup_cause();

	switch (wakeup_reason) {
	case ESP_SLEEP_WAKEUP_EXT0:
		ESP_LOGD(TAG.c_str(), "Wakeup caused by external signal using RTC_IO");
		break;
	case ESP_SLEEP_WAKEUP_EXT1:
		ESP_LOGD(TAG.c_str(), "Wakeup caused by external signal using RTC_CNTL");
		break;
	case ESP_SLEEP_WAKEUP_TIMER:
		ESP_LOGD(TAG.c_str(), "Wakeup caused by timer");
		break;
	case ESP_SLEEP_WAKEUP_TOUCHPAD:
		ESP_LOGD(TAG.c_str(), "Wakeup caused by touchpad");
		break;
	case ESP_SLEEP_WAKEUP_ULP:
		ESP_LOGD(TAG.c_str(), "Wakeup caused by ULP program");
		break;
	default:
		ESP_LOGD(TAG.c_str(), "Wakeup was not caused by deep sleep: %d", wakeup_reason);
		break;
	}
}

void deep_sleep(int time_in_sec)
{
	esp_sleep_enable_timer_wakeup(time_in_sec * 1000000);
	esp_deep_sleep_start();
}

}; // namespace DeepSleep
