#include "Message.hpp"

static const char *TAG = "MESSAGE";

esp_err_t Message::send() const {
  ESP_LOGD(TAG, "Sending message");
  esp_err_t success;

  // conversion from std::string to c_str adds null terminator, which is why we add 1 to message length
  success = esp_now_send(recipient,
                         (uint8_t *) &clientDataPackage,
                         sizeof(ClientDataPackage));
  if (success != ESP_OK) {
    ESP_LOGE(TAG, "Error sending the data");
    // Removed caching from here, better do this in main
  }
//    ESP_LOGD(TAG, "Sent data: %s", messageData.c_str());

  ESP_LOGD(TAG, "Timestamp sent: %ld", clientDataPackage.getTimestamp());
  ESP_LOGD(TAG, "send status: %d", success);

  return success;
}

esp_err_t Message::sendMessages(const std::array<Message, 6> &messages) {
  // recipient
  uint8_t rec[6]{};
  get_host_mac(rec);
  ESP_LOGD(TAG, "Sending messages");
  esp_err_t success;
  // list of compressed data. Sending 6 at a time
  std::array<ClientDataPackage, 6> clientDataPackages{};
  // max 6 messages
  int i = 0;
  for (const auto &message: messages) {
    if (i >= 6) {
      ESP_LOGE(TAG, "Too many messages to send");
      break;
    }
    clientDataPackages[i] = message.getClientDataPackage();

    i++;
  }

  // conversion from std::string to c_str adds null terminator, which is why we add 1 to message length
  success = esp_now_send(rec, (uint8_t *) &clientDataPackages,
                         sizeof(std::array<ClientDataPackage, 6>));
  if (success != ESP_OK) {
    ESP_LOGE(TAG, "Error sending the data");
    // Removed caching from here, better do this in main
  }
  unsigned long ts = millis();
  // it takes ~110ms for receiving an acknowledgement by the host in perfect conditions
  uint16_t message_timeout = 2000;
  while (!was_msg_received()) {
    if ((millis() - ts) > message_timeout) {
      ESP_LOGE(TAG, "Timeout: Host not available\n");
      break;
    }
  }
  ESP_LOGD(TAG, "Time until acknowledgement: %ld", millis() - ts);
//    ESP_LOGD(TAG, "Sent data: %s", messageData.c_str());

//    ESP_LOGD(TAG, "Timestamp sent: %ld", clientDataPackage.getTimestamp());
  ESP_LOGD(TAG, "send status: %d", success);

  return success;
}

std::string Message::getMessageAsMinifiedJsonString() const {
  return clientDataPackage.getDataPackageAsMinifiedJsonString();
}

Message::Message(ClientDataPackage data) : clientDataPackage(data) {
  // check for existing host mac address, use broadcast otherwise
  get_host_mac(recipient);
}

Message::Message() {
  // check for existing host mac address, use broadcast otherwise
  get_host_mac(recipient);
}

Message::Message(Measurement const &data,
                 const SensorInformation &information,
                 unsigned long timestamp)
    : clientDataPackage(data, information, timestamp) {
  // check for existing host mac address, use broadcast otherwise
  get_host_mac(recipient);
}
const ClientDataPackage &Message::getClientDataPackage() const {
  return clientDataPackage;
}

