#pragma once

#include "ClientDataPackage.hpp"
#include "ESPNow.hpp"
#include "Time.hpp"
#include "esp_log.h"
#include "GlobalDefinitions.hpp"
#include <Arduino.h>
#include <ArduinoJson.h>
#include <ESP32Time.h>
#include <esp_now.h>
#include <utility>

// Format of the message sent from host to client_satellite
// if more things are sent from the host the name might not be accurate anymore
class Message {
public:
    Message();
    explicit Message(ClientDataPackage data);

    Message(Measurement const &data,
            const SensorInformation &information,
            unsigned long timestamp);

    esp_err_t send() const;

    [[nodiscard]] std::string getMessageAsMinifiedJsonString() const;

    static Message nullMessage() {
        return Message{Measurement(ERROR_VALUE,
                                   NO_CHANNEL,
                                   NO_I2C_ADDRESS,
                                   MeasurementType::NULL_MEASUREMENT, ErrorType::NULL_MESSAGE),
                       SensorInformation(HardwareName::NONE,
                                         SensorProtocol::NULL_PROTOCOL),
                       NULL_TIMESTAMP};
    }

    static esp_err_t sendMessages(const std::array<Message, 6> &messages);

    [[nodiscard]] const ClientDataPackage &getClientDataPackage() const;

private:
    ClientDataPackage clientDataPackage;

    uint8_t recipient[6]{};

};
