#include "scd30.hpp"

void ForteSCD30::setup() {
    Wire.begin(I2C_SDA, I2C_SCL);
    if (!airSensor.begin()) {
        // Sensor init went wrong
        ESP_LOGW(sensorInformation.getHardwareNameString().c_str(),
                 "Initialization failed.");
        sensorInit = false;
    } else
        sensorInit = true;
}

out_data_scd30 ForteSCD30::readData() {
    if (airSensor.dataAvailable()) {
        data.C02 = Measurement{static_cast<double>(airSensor.getCO2()), NO_CHANNEL,
                               SCD30_BASE_ADDRESS,
                               MeasurementType::CO2, ErrorType::DATA_OK};

        data.Temperature = Measurement{airSensor.getTemperature(), NO_CHANNEL, SCD30_BASE_ADDRESS,
                                       MeasurementType::TEMPERATURE, ErrorType::DATA_OK};
        data.Humidity = Measurement{airSensor.getHumidity(), NO_CHANNEL, SCD30_BASE_ADDRESS,
                                    MeasurementType::HUMIDITY, ErrorType::DATA_OK};
    } else {
        data.C02 = Measurement{ERROR_VALUE, NO_CHANNEL, SCD30_BASE_ADDRESS,
                               MeasurementType::CO2, ErrorType::NO_DATA};
        data.Temperature = Measurement{ERROR_VALUE, NO_CHANNEL, SCD30_BASE_ADDRESS,
                                       MeasurementType::TEMPERATURE, ErrorType::NO_DATA};
        data.Humidity = Measurement{ERROR_VALUE, NO_CHANNEL, SCD30_BASE_ADDRESS,
                                    MeasurementType::HUMIDITY, ErrorType::NO_DATA};
    }

    if (!sensorInit) {
        data.C02.setErrorType(ErrorType::SENSOR_INIT_FAILED);
        data.Temperature.setErrorType(ErrorType::SENSOR_INIT_FAILED);
        data.Humidity.setErrorType(ErrorType::SENSOR_INIT_FAILED);
    }

    return data;
}

std::list<Message> ForteSCD30::buildMessages() {
    std::list<Message> messages;

    auto measurements = readData();


    messages.emplace_back(measurements.C02,
                          sensorInformation,
                          Time::getInstance().getEpochSeconds());
    messages.emplace_back(measurements.Temperature,
                          sensorInformation,
                          Time::getInstance().getEpochSeconds());
    messages.emplace_back(measurements.Humidity,
                          sensorInformation,
                          Time::getInstance().getEpochSeconds());

    return messages;
}

SensorInformation ForteSCD30::getSensorInformation() const {
    return sensorInformation;
}
